#define BAUD_ 9600
#define F_CPU 8000000 // 8 MHz

#include <util/delay.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>

uint16_t tempADC = 0;
uint8_t byte1 = 0;
uint8_t byte2 = 0;

uint8_t byteA = 0;
uint8_t byteB = 0;
uint8_t byteC = 0;
uint8_t byteD = 0;


uint8_t pwmData[4] = {0}; //2 16-bit numbers = 4 8-bit bytes

uint16_t readADC(uint8_t ch)
{
    // select the corresponding channel 0~7
    // ANDing with '7' will always keep the value
    // of 'ch' between 0 and 7
    ch &= 0b00000111;  // AND operation with 7
    ADMUX = (ADMUX & 0xF8)|ch;     // clears the bottom 3 bits before ORing
  
    // start single conversion
    // write '1' to ADSC
    ADCSRA |= (1<<ADSC);
  
    // wait for conversion to complete
    // ADSC becomes '0' again
    // till then, run loop continuously
    while(ADCSRA & (1<<ADSC));
  
    return (ADC);
}

void transmit(char data)
{
  while( !(UCSR0A & (1 << UDRE0))); // Wait until Tx is ready
  UDR0 = data;
}

char receive(void)
{
  while(!(UCSR0A & (1 << RXC0)));
  return UDR0;
}

void updatePWM() {

  //update OCR1A & OCR1B 16-bit data from uart rx buffer
  //for (uint8_t i = 0; i < 4; i++) {
  //pwmData[i] = receive();
  //}

  
  uint8_t tempUART = receive();
  uint16_t tempPWM = (tempUART << 8);
  tempUART = receive();
  tempPWM |= tempUART;

  OCR1A = tempPWM;
}

int main()
{
  //Setup ADC
  // AREF = AVcc
  ADMUX = (1<<REFS0);
  
  // ADC Enable and prescaler of 128
  // 16000000/128 = 125000
  ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
       
  
  //Setup UART for 9600 baud
  UBRR0 = 51;
  UCSR0B |= (1 << TXEN0)|(1 << RXEN0); //enable tx & rx
  UCSR0C |= (1 << UCSZ00)|(1 << UCSZ01); // set 8-bit data


  //Enable PWM
  DDRB |= (1 << DDB1)|(1 << DDB2);
  //PB1 is PWM1 & PB2 is PWM2

  ICR1 = 0xFFFF;
  // set TOP to 16bit

  //OCR1A = 0x3FFF; // set PWM for 25% duty cycle @ 16bit
  //OCR1A = 0xBFFF;
  //OCR1A = 0xFFFF;
  //OCR1A = 0x0000;
  OCR1A = 32000;

  //OCR1B = 0x3FFF; // set PWM for 25% duty cycle @ 16bit
  //OCR1B = 0xBFFF; // set PWM for 75% duty cycle @ 16bit
  //OCR1B = 0b0000111111111111;
  //OCR1B = 0xFFFF;
  OCR1B = 32000;

  TCCR1A |= (1 << COM1A1)|(1 << COM1B1);
  // set none-inverting mode

  TCCR1A |= (1 << WGM11);
  TCCR1B |= (1 << WGM12)|(1 << WGM13);
  // set Fast PWM mode using ICR1 as TOP
    
  TCCR1B |= (1 << CS10);
  // START the timer with no prescaler
       
  
  //sei(); //Enable Global Interrupts
  
 
  //Setup variable for uart data
  char input;
  
  while(1)
    {

  


  
  
      input = receive();
      
      if (input == 'a') {
	transmit('x');
	//OCR1A = 0;
	//OCR1B = 65000;
	for (int i = 0; i < 10; i++) {
	  OCR1A = 6000*i;
	  _delay_ms( 100 );
	  //OCR1B = 6000*i;
	}
	/*	for (int i = 0; i < 10; i++) {
	  OCR1A = 6000*i;
	  //OCR1B = i;
	}
	*/ 
      }
      
      else if (input == 'b') {
	transmit('y');
	OCR1A = 32000;
	OCR1B = 32000;


      }

      //program anemone response- rapidly deflate, then slowly pump back up
      else if (input == 'c') {
	transmit('z');
	//OCR1A = 65000;
	//OCR1B = 0;
	for (int i = 10; 0 < i; i--) {
	  OCR1B = 6000*i;
	  _delay_ms( 100 );
	  //OCR1B = 6000*i;
	}
	
      }

      else if (input == 'd') {
	transmit('l');
	//deflate both anemones
	OCR1B = 65000;
	_delay_ms(150);
	OCR1A = 65000;
	for (int i = 10; 0 < i; i--) {
	  OCR1B = 6000*i;
	  _delay_ms( 100 );
	}
	for (int i = 10; 0 < i; i--) {
	  OCR1A = 6000*i;
	  _delay_ms( 100 );
	}
	
      }

      else if (input == 'e') {
	transmit('m');
	OCR1A = 0;
	OCR1B = 0;
      }
      

      else if (input == 'f') {
	transmit('1');
	updatePWM();
}

      else if (input == 'g') {

  for (int i = 0; i < 4; i++) {
  pwmData[i] = receive();
  tempADC = readADC(i);
  byte1 = (tempADC >> 2);
  transmit(byte1);
}
}
	else if (input == 'h') {
		transmit('o');
		OCR1A = 31000;
		//OCR1B = 32000;
 
  
  
}

	else if (input == 'n') {
		transmit('l');
		OCR1A = 30000;
		//OCR1B = 31000;
}
      

	else if (input == 'u') {
		transmit('i');
		OCR1B = 34000;
}

	else if (input == 'j') {
		transmit('k');
		OCR1B = 33000;
}

      // Need to send responses when receive UART data

	else if (input == 'l') {
	  transmit('m');

	  //send LSB first
	  //read two bytes, shift and set to OCR1A
	  byteA = receive();
	  transmit('1');
	  byteB = receive();
	  transmit('2');
	  OCR1B = (byteB << 8) | byteA;
	  
	  //read two bytes, shift and set to OCR1B
	  //byteC = receive();
	  //byteD = receive();
	  //OCR1B = (byteD << 8) | byteC;
	}

      else if (input == 'r') {
	  transmit('s');

	  //send LSB first
	  //read two bytes, shift and set to OCR1A
	  //byteA = receive();
	  //byteB = receive();
	  //OCR1A = (byteB << 8) | byteA;
	  
	  //read two bytes, shift and set to OCR1B
	  byteC = receive();
	  transmit('3');
	  byteD = receive();
	  transmit('4');
	  OCR1A = (byteD << 8) | byteC;
	}
      
      else
	{
	  
	  transmit('0');
	  
	  
	} 	  
    
  

}
  
  return 0;
}

